import { Button, DialogContent, FormControl, InputLabel, MenuItem, Select } from '@mui/material';
import PreparedDialog from 'components/common/Dialog/PreparedDialog';
import Form from 'components/common/Form/Form';
import InputForm from 'components/common/Input/InputForm';
import useAxiosPrivate from 'hooks/useAxiosPrivate';
import { useEffect, useState } from 'react';
import { useNavigate } from 'react-router-dom';
import * as backendUrl from 'services/backendUrl';
import { displaySnackbar } from 'utils/snackbar';

// FIXME: refactoring
export default function InvitationFormModal({ isOpenModal, setIsOpenModal }) {
    const navigate = useNavigate();
    const axiosPrivate = useAxiosPrivate();
    const [mentorEmail, setMentorEmail] = useState('');
    const [selectedMentor, setSelectedMentor] = useState(undefined);
    const [foundMentors, setFoundMentors] = useState([]);
    const [open, setOpen] = useState(false);
    const defaultTimeoutMsec = 2500;

    const handleOpen = () => setOpen(true);
    const handleClose = () => {
        setIsOpenModal(false);
        setOpen(false);
        setFoundMentors([]);
        setSelectedMentor('');
    };

    useEffect(() => {
        if (isOpenModal) {
            handleOpen();
        }
    }, [isOpenModal]);

    const handleClickOnSendButton = (event) => {
        event.preventDefault();
        if (!selectedMentor) {
            const message = 'Тренер не выбран';
            const snackbarKey = displaySnackbar(message, 'error');
            return;
        }
        axiosPrivate
            .post(`${backendUrl.sendInvitationUrl}/${selectedMentor}`)
            .then((response) => {
                if (response.status === 201) {
                    const message = 'Запрос на приглашение успешно отправлен';
                    const snackbarKey = displaySnackbar(message, 'success');
                    setTimeout(() => {
                        navigate('/account/clients', { replace: false });
                    }, defaultTimeoutMsec);
                }
            })
            .catch((error) => {
                if (error.response && error.response.status === 403) {
                    const message = 'Нельзя отправить запрос на присоединение самому себе';
                    const snackbarKey = displaySnackbar(message, 'error');
                } else if (error.response && error.response.status === 400) {
                    const message = 'Вы уже отправили запрос данному тренеру';
                    const snackbarKey = displaySnackbar(message, 'error');
                } else if (error.request) {
                    const message = 'Произошла ошибка при отправке запроса';
                    const snackbarKey = displaySnackbar(message, 'error');
                } else {
                    console.log('Error', error.message);
                }
            });
    };

    const handleClickOnSearchButton = () => {
        if (mentorEmail === '') {
            const message = 'Введите email тренера';
            const snackbarKey = displaySnackbar(message, 'error');
            return;
        }
        axiosPrivate
            .post(`${backendUrl.searchMentorsUrl}`, {
                email: mentorEmail
            })
            .then((response) => {
                if (response.data.length === 0) {
                    const message = 'Тренеры не были найдены :(';
                    const snackbarKey = displaySnackbar(message, 'error');
                    return;
                }
                setFoundMentors(response.data);
                const message = 'Запрос выполнен. Выберите тренеров из списка';
                const snackbarKey = displaySnackbar(message, 'success');
            })
            .catch((error) => {
                const message = 'Вы ввели некорректный email';
                const snackbarKey = displaySnackbar(message, 'error');
            });
    };

    const handleChange = (event) => {
        setSelectedMentor(event.target.value);
    };

    return (
        <PreparedDialog
            open={open}
            onClose={handleClose}
            scroll="body"
            aria-labelledby="scroll-dialog-title"
            aria-describedby="scroll-dialog-description">
            <DialogContent
                dividers={scroll === 'paper'}
                className="flex justify-center items-center p-0">
                <div className="flex flex-col items-center bg-[#e3f2fc] shadow-md rounded-lg gap-2 py-3 w-full">
                    <span className="font-bold text-[var(--color-unused-text-gray)] text-lg text-center px-4 text-nowrap">
                        Заполните форму
                    </span>
                    <Form onSubmit={handleClickOnSendButton} className="flex flex-col gap-y-3">
                        <div className="flex flex-row justify-center items-center gap-x-1 px-2 lg:px-0">
                            <InputForm
                                className="text-md bg-[#fefefe] p-2 border-solid border-[#a3a9b2] border-2 rounded-md w-full"
                                type="text"
                                placeholder="Найдите тренера (по email)"
                                required
                                onChange={(e) => setMentorEmail(e.target.value)}
                            />
                        </div>
                        <div className="px-2 lg:px-0">
                            <FormControl sx={{ width: '100%' }}>
                                <InputLabel id="select-autowidth-label">Тренер</InputLabel>
                                <Select
                                    labelId="select-autowidth-label"
                                    id="select-autowidth"
                                    value={selectedMentor}
                                    onChange={(event) => handleChange(event)}
                                    autoWidth
                                    label="Mentor">
                                    <MenuItem value="" className="text-lg lg:text-lg">
                                        <em>Не выбран</em>
                                    </MenuItem>
                                    {foundMentors.length > 0 ? (
                                        foundMentors.map((mentor) => (
                                            <MenuItem
                                                value={mentor.id}
                                                key={mentor.id}
                                                className="text-lg lg:text-lg">
                                                {mentor.email}
                                            </MenuItem>
                                        ))
                                    ) : (
                                        <></>
                                    )}
                                </Select>
                            </FormControl>
                        </div>
                        <div className="flex px-1 lg:px-0 justify-center">
                            {foundMentors.length !== 0 ? (
                                <Button
                                    className="flex flex-row justify-center font-normal rounded-xl border-solid border-2 py-2 px-4 lg:px-10 hover:text-[var(--color-form-white)] hover:border-[var(--color-thymeleaf)] hover:bg-[var(--color-thymeleaf)] text-[var(--color-form-white)] bg-[var(--color-thymeleaf)] border-[var(--color-thymeleaf)] normal-case"
                                    type="submit">
                                    Отправить запрос
                                </Button>
                            ) : (
                                <Button
                                    className="flex flex-row justify-center font-normal rounded-xl border-solid border-2 py-2 px-4 lg:px-10 hover:text-[var(--color-form-white)] hover:border-[var(--color-thymeleaf)] hover:bg-[var(--color-thymeleaf)] text-[var(--color-form-white)] bg-[var(--color-thymeleaf)] border-[var(--color-thymeleaf)] normal-case text-xl"
                                    onClick={() => handleClickOnSearchButton()}
                                    type="button">
                                    Найти тренеров
                                </Button>
                            )}
                        </div>
                    </Form>
                </div>
            </DialogContent>
        </PreparedDialog>
    );
}
