import { CheckRounded, ClearRounded, ExpandMore } from '@mui/icons-material';
import DeleteForeverIcon from '@mui/icons-material/DeleteForever';
import EditIcon from '@mui/icons-material/Edit';
import {
    Accordion,
    AccordionDetails,
    AccordionSummary,
    Button,
    DialogContent,
    IconButton,
    InputBase,
    Typography
} from '@mui/material';
import { blue, green, red } from '@mui/material/colors';
import CustomDivider from 'components/common/CustomDivider/CustomDivider';
import PreparedDialog from 'components/common/Dialog/PreparedDialog';
import PropertyHandledValueFlexContainer from 'components/common/PropertyHandledValueFlexContainer';
import PropertyValueFlexContainer from 'components/common/PropertyValueFlexContainer';
import { useEffect } from 'react';
import { useNavigate } from 'react-router-dom';
import { axiosPrivate } from 'services/axios';
import * as backendUrl from 'services/backendUrl';
import { useImmer } from 'use-immer';
import { displaySnackbar } from 'utils/snackbar';

const defaultTimeoutMsec = 2000;

// TODO: specify props types
interface ClientInfoModalProps {
    readonly selectedUser: any;
    readonly setSelectedUser: any;
    readonly isMentor: boolean;
}

export default function ClientInfoModal({
    selectedUser,
    setSelectedUser,
    isMentor
}: ClientInfoModalProps) {
    const navigate = useNavigate();
    const [isModalWindowOpen, setIsModalWindowOpen] = useImmer(false);
    const [isEditPanel, setIsEditPanel] = useImmer(false);
    const [inputDescription, setInputDescription] = useImmer(
        selectedUser ? selectedUser.description : ''
    );

    const handleClose = () => {
        setIsModalWindowOpen(false);
        setSelectedUser(undefined);
    };

    useEffect(() => {
        if (selectedUser) {
            setIsModalWindowOpen(true);
        }
    }, [selectedUser]);

    const handleClickOnDeleteButton = () => {
        axiosPrivate
            .delete(`${backendUrl.deleteClientUrl}/${selectedUser.id}`)
            .then((response) => {
                if (response.status === 204) {
                    const message = 'Клиент успешно удален из списка';
                    const snackbarKey = displaySnackbar(message, 'success');
                    setTimeout(() => {
                        navigate('/account/clients', { replace: false });
                    }, defaultTimeoutMsec);
                }
            })
            .catch((error) => {
                const message = 'Произошла ошибка при отправке запроса';
                const snackbarKey = displaySnackbar(message, 'error');
            });
    };

    const handleClickOnCheckButton = () => {
        if (inputDescription === selectedUser.description) {
            setIsEditPanel(false);
            return;
        }
        axiosPrivate
            .patch(backendUrl.BACKEND_URL + backendUrl.clientsUrl + `/${selectedUser.id}`, {
                description_from_mentor: inputDescription
            })
            .then((response) => {
                if (response.status === 204) {
                    window.location.reload();
                    const message = 'Описание клиента обновлено';
                    const snackbarKey = displaySnackbar(message, 'success');
                }
            })
            .catch((error) => {
                console.log(error);
            });
        setIsEditPanel(!isEditPanel);
    };

    return (
        <>
            {selectedUser ? (
                <PreparedDialog
                    open={isModalWindowOpen}
                    onClose={handleClose}
                    scroll="body"
                    aria-labelledby="scroll-dialog-title"
                    aria-describedby="scroll-dialog-description">
                    <DialogContent
                        dividers={scroll === 'paper'}
                        className="flex justify-center items-center p-0">
                        <div className="flex flex-col items-center bg-[#e3f2fc] shadow-md rounded-lg gap-2 py-3 w-full">
                            <span className="font-bold text-[var(--color-unused-text-gray)] text-3xl text-center px-4 text-nowrap">
                                Информация о клиенте
                            </span>
                            <div className="text-2xl md:text-lg bg-[var(--color-contrast-blue)] text-[var(--color-form-white)] rounded-xl px-2 py-3 mb-1 mx-2">
                                <div className="flex flex-col justify-center lg:mx-6">
                                    <PropertyValueFlexContainer
                                        property="Имя:"
                                        value={selectedUser.first_name}
                                    />
                                    <PropertyValueFlexContainer
                                        property="Фамилия:"
                                        value={selectedUser.last_name}
                                    />
                                    <PropertyValueFlexContainer
                                        property="Дата рождения:"
                                        value={selectedUser.birthday}
                                    />
                                    <PropertyHandledValueFlexContainer
                                        property="Пол:"
                                        rawValue={selectedUser.sex}
                                        targetValue="MALE"
                                        handledValue="мужской"
                                        alternativeHandledValue="женский"
                                    />
                                    <PropertyValueFlexContainer
                                        property="Email:"
                                        value={selectedUser.email}
                                    />
                                    {!isMentor && (
                                        <>
                                            <CustomDivider />
                                            <Accordion
                                                sx={{ p: 0 }}
                                                className="text-2xl md:text-lg bg-[var(--color-contrast-blue)] text-[var(--color-form-white)]">
                                                <AccordionSummary
                                                    sx={{ p: 0 }}
                                                    expandIcon={<ExpandMore />}
                                                    aria-controls="panel1-content"
                                                    id="panel1-header">
                                                    <Typography component="span" variant="h5">
                                                        Заметка о клиенте
                                                    </Typography>
                                                </AccordionSummary>
                                                <AccordionDetails>
                                                    <div className="flex flex-row flex-wrap bg-[var(--color-background-light)] text-[var(--color-contrast-blue)] rounded-lg py-1 px-3 justify-center items-center">
                                                        {isEditPanel ? (
                                                            <>
                                                                <InputBase
                                                                    placeholder={
                                                                        selectedUser.description
                                                                    }
                                                                    defaultValue={
                                                                        selectedUser.description
                                                                    }
                                                                    className="text-[var(--color-form-dark-black)] text-lg pl-1 lg:pr-4 bg-slate-500 bg-opacity-15 rounded-sm"
                                                                    onChange={(e) =>
                                                                        setInputDescription(
                                                                            e.target.value
                                                                        )
                                                                    }
                                                                />
                                                                <IconButton
                                                                    sx={{ color: red[400] }}
                                                                    size="medium"
                                                                    onClick={() =>
                                                                        setIsEditPanel(false)
                                                                    }>
                                                                    <ClearRounded />
                                                                </IconButton>
                                                                <IconButton
                                                                    sx={{ color: green[700] }}
                                                                    size="medium"
                                                                    onClick={() =>
                                                                        handleClickOnCheckButton()
                                                                    }>
                                                                    <CheckRounded />
                                                                </IconButton>
                                                            </>
                                                        ) : (
                                                            <>
                                                                <span className="text-[var(--color-form-dark-black)] text-lg pl-1 pr-4">
                                                                    {selectedUser.description ??
                                                                        'Не указано'}
                                                                </span>
                                                                <IconButton
                                                                    sx={{ color: blue[700] }}
                                                                    size="medium"
                                                                    onClick={() =>
                                                                        setIsEditPanel(!isEditPanel)
                                                                    }>
                                                                    <EditIcon />
                                                                </IconButton>
                                                                <IconButton
                                                                    sx={{ color: red[400] }}
                                                                    size="medium">
                                                                    <DeleteForeverIcon />
                                                                </IconButton>
                                                            </>
                                                        )}
                                                        {/* <div className="flex flex-row">
                                                            {isEditPanel ? (
                                                                <>
                                                                    <IconButton
                                                                        sx={{ color: green[700] }}
                                                                        size="medium"
                                                                        onClick={() =>
                                                                            handleClickOnCheckButton()
                                                                        }>
                                                                        <CheckRounded />
                                                                    </IconButton>
                                                                    <IconButton
                                                                        sx={{ color: red[400] }}
                                                                        size="medium"
                                                                        onClick={() =>
                                                                            setIsEditPanel(false)
                                                                        }>
                                                                        <ClearRounded />
                                                                    </IconButton>
                                                                </>
                                                            ) */}
                                                        {/* </div>
                                                    </>
                                                ) */}
                                                    </div>
                                                </AccordionDetails>
                                            </Accordion>
                                        </>
                                    )}
                                </div>
                            </div>
                            <Button
                                className="bg-[var(--color-delete-for-button)] rounded-2xl text-2xl font-normal text-[var(--color-form-white)] px-6 py-1 mx-2 mb-1"
                                onClick={(e) => handleClickOnDeleteButton()}>
                                удалить
                            </Button>
                        </div>
                    </DialogContent>
                </PreparedDialog>
            ) : (
                <></>
            )}
        </>
    );
}
