import {
    Box,
    Button,
    Chip,
    DialogContent,
    FormControl,
    InputLabel,
    MenuItem,
    OutlinedInput,
    Select
} from '@mui/material';
import PreparedDialog from 'components/common/Dialog/PreparedDialog';
import Form from 'components/common/Form/Form';
import InputForm from 'components/common/Input/InputForm';
import useAuth from 'hooks/useAuth';
import useAxiosPrivate from 'hooks/useAxiosPrivate';
import { useEffect, useState } from 'react';
import { useNavigate } from 'react-router-dom';
import * as backendUrl from 'services/backendUrl';
import styled from 'styled-components';

import { equipmentList } from 'data/mockEquipments';
import { metricsList } from 'data/mockMetrics';
import { parametersList } from 'data/mockParameters';
import { displaySnackbar } from 'utils/snackbar';

const ITEM_HEIGHT = 48;
const ITEM_PADDING_TOP = 8;
const MenuProps = {
    PaperProps: {
        style: {
            maxHeight: ITEM_HEIGHT * 4.5 + ITEM_PADDING_TOP,
            width: 250
        }
    }
};

const CustomOutlinedInput = styled(OutlinedInput)(({ theme }) => ({
    '& .MuiOutlinedInput-notchedOutline legend': {
        width: 'max-content'
    }
}));

export default function ExerciseCreationModal({ isOpenModal, setIsOpenModal }) {
    const navigate = useNavigate();
    const axiosPrivate = useAxiosPrivate();
    const [open, setOpen] = useState(false);
    const [exerciseName, setExerciseName] = useState('');
    const [metricsMetaInfo, setMetricsMetaInfo] = useState([]);
    const [parametersMetaInfo, setParametersMetaInfo] = useState([]);
    const [description, setDescription] = useState('');
    const [mediaLinks, setMediaLinks] = useState([]);
    const [equipments, setEquipments] = useState([]);
    const { auth } = useAuth();
    const defaultTimeoutMsec = 2500;

    const handleOpen = () => setOpen(true);
    const handleClose = () => {
        setIsOpenModal(false);
        setOpen(false);
        setEquipments([]);
        setParametersMetaInfo([]);
        setMetricsMetaInfo([]);
    };

    useEffect(() => {
        if (isOpenModal) {
            handleOpen();
        }
    }, [isOpenModal]);

    const handleClickOnButton = (event) => {
        event.preventDefault();
        const selectedEquipments = equipmentList.filter((equipment) => {
            return equipments.some((meta) => {
                const [name, unit] = meta.split(', ').map((str) => str.trim());
                return equipment.name === name && equipment.unit === unit;
            });
        });

        const selectedMetrics = metricsList.filter((metric) => {
            return metricsMetaInfo.some((meta) => {
                const [name, unit] = meta.split(', ').map((str) => str.trim());
                return metric.name === name && metric.unit === unit;
            });
        });

        const selectedParameters = parametersList.filter((parameter) => {
            return parametersMetaInfo.some((meta) => {
                const [name, unit] = meta.split(', ').map((str) => str.trim());
                return parameter.name === name && parameter.unit === unit;
            });
        });

        axiosPrivate
            .post(`${backendUrl.exercisesUrl}`, {
                name: exerciseName,
                description: description,
                media: mediaLinks,
                equipments: selectedEquipments,
                metrics_template: selectedMetrics,
                parameters_template: selectedParameters
            })
            .then((response) => {
                if (response.status === 201) {
                    const message = 'Упражнение успешно создано';
                    const snackbarKey = displaySnackbar(message, 'success');
                    // FIXME
                    setTimeout(() => {
                        navigate('/account/exercises', { replace: false });
                    }, defaultTimeoutMsec);
                }
            })
            .catch((error) => {
                const message = 'Произошла ошибка при отправке запроса';
                const snackbarKey = displaySnackbar(message, 'success');
                console.log('Request Error', error);
            });
    };

    const handleEquipmentChange = (event) => {
        const {
            target: { value }
        } = event;
        setEquipments(typeof value === 'string' ? value.split(',') : value);
    };

    const handleMetricChange = (event) => {
        const {
            target: { value }
        } = event;
        setMetricsMetaInfo(typeof value === 'string' ? value.split(',') : value);
    };

    const handleParameterChange = (event) => {
        const {
            target: { value }
        } = event;
        setParametersMetaInfo(typeof value === 'string' ? value.split(',') : value);
    };

    return (
        <PreparedDialog
            open={open}
            onClose={handleClose}
            scroll="body"
            aria-labelledby="scroll-dialog-title"
            aria-describedby="scroll-dialog-description">
            <DialogContent
                dividers={scroll === 'paper'}
                className="flex justify-center items-center p-0">
                <div className="flex flex-col items-center bg-[#e3f2fc] shadow-md rounded-lg gap-2 py-3 w-full">
                    <span className="font-bold text-[var(--color-unused-text-gray)] text-3xl text-center px-4">
                        Добавление упражнения
                    </span>
                    <Form
                        onSubmit={handleClickOnButton}
                        className="flex flex-column flex-wrap justify-start items-center gap-3 px-3">
                        <InputForm
                            className="text-lg lg:text-2xl bg-[#fefefe] p-2 border-solid border-[#a3a9b2] border-2 rounded-md w-full"
                            type="text"
                            placeholder="Название упражнения"
                            required
                            onChange={(e) => setExerciseName(e.target.value)}
                        />
                        <InputForm
                            className="text-lg lg:text-2xl bg-[#fefefe] p-2 border-solid border-[#a3a9b2] border-2 rounded-md w-full"
                            type="text"
                            placeholder="Описание упражнения"
                            onChange={(e) => setDescription(e.target.value)}
                        />
                        {/* FIXME: error here */}
                        <FormControl sx={{ minWidth: '100%' }}>
                            <InputLabel
                                id="multiple-chip-label"
                                sx={{
                                    m: 0,
                                    minWidth: '300'
                                }}>
                                Снаряжение
                            </InputLabel>
                            <Select
                                labelId="multiple-chip-label"
                                id="multiple-chip"
                                multiple
                                value={equipments}
                                onChange={handleEquipmentChange}
                                input={<CustomOutlinedInput label="Снаряжение" sx={{ m: 0 }} />}
                                renderValue={(selected) => (
                                    <Box
                                        key={selected.value}
                                        sx={{ display: 'flex', flexWrap: 'wrap', gap: 0.5 }}>
                                        {selected.map((value) => (
                                            <Chip key={value} label={value} />
                                        ))}
                                    </Box>
                                )}
                                MenuProps={MenuProps}>
                                {equipmentList.map((equipment) => (
                                    <MenuItem key={equipment.id} value={equipment.name}>
                                        {equipment.name}
                                    </MenuItem>
                                ))}
                            </Select>
                        </FormControl>
                        <FormControl sx={{ minWidth: '100%' }}>
                            <InputLabel
                                id="multiple-chip-label"
                                sx={{
                                    m: 0,
                                    minWidth: 300
                                }}>
                                Метрики
                            </InputLabel>
                            <Select
                                labelId="multiple-chip-label"
                                id="multiple-chip"
                                multiple
                                value={metricsMetaInfo}
                                onChange={handleMetricChange}
                                input={<CustomOutlinedInput label="Метрики" sx={{ m: 0 }} />}
                                renderValue={(selected) => (
                                    <Box sx={{ display: 'flex', flexWrap: 'wrap', gap: 0.5 }}>
                                        {selected.map((value) => (
                                            <Chip key={value} label={value} />
                                        ))}
                                    </Box>
                                )}
                                MenuProps={MenuProps}>
                                {metricsList.map((metric) => (
                                    <MenuItem
                                        key={metric.name}
                                        value={metric.name + ', ' + metric.unit}>
                                        {metric.name + ', ' + metric.unit}
                                    </MenuItem>
                                ))}
                            </Select>
                        </FormControl>
                        <FormControl sx={{ minWidth: '100%' }}>
                            <InputLabel
                                id="multiple-chip-label"
                                sx={{
                                    m: 0,
                                    minWidth: 300
                                }}>
                                Параметры
                            </InputLabel>
                            <Select
                                labelId="multiple-chip-label"
                                id="multiple-chip"
                                multiple
                                value={parametersMetaInfo}
                                onChange={handleParameterChange}
                                input={<CustomOutlinedInput label="Параметры" sx={{ m: 0 }} />}
                                renderValue={(selected) => (
                                    <Box sx={{ display: 'flex', flexWrap: 'wrap', gap: 0.5 }}>
                                        {selected.map((value) => (
                                            <Chip key={value} label={value} />
                                        ))}
                                    </Box>
                                )}
                                MenuProps={MenuProps}>
                                {parametersList.map((parameter) => (
                                    <MenuItem
                                        key={parameter.name}
                                        value={parameter.name + ', ' + parameter.unit}>
                                        {parameter.name + ', ' + parameter.unit}
                                    </MenuItem>
                                ))}
                            </Select>
                        </FormControl>
                        <Button
                            className="flex flex-row justify-center font-normal rounded-xl border-solid border-2 py-2 px-4 lg:px-10 hover:text-[var(--color-form-white)] hover:border-[var(--color-thymeleaf)] hover:bg-[var(--color-thymeleaf)] text-[var(--color-form-white)] bg-[var(--color-thymeleaf)] border-[var(--color-thymeleaf)] w-full text-xl"
                            type="submit">
                            Создать упражнение
                        </Button>
                    </Form>
                </div>
            </DialogContent>
        </PreparedDialog>
    );
}
