import {
    Accordion,
    AccordionDetails,
    AccordionSummary,
    Box,
    Button,
    DialogContent,
    FormControl,
    InputLabel,
    MenuItem,
    Select
} from '@mui/material';
import { AdapterDayjs } from '@mui/x-date-pickers/AdapterDayjs';
import { LocalizationProvider } from '@mui/x-date-pickers/LocalizationProvider';
import PreparedDialog from 'components/common/Dialog/PreparedDialog';
import Form from 'components/common/Form/Form';
import InputForm from 'components/common/Input/InputForm';
import useAxiosPrivate from 'hooks/useAxiosPrivate';
import { useEffect } from 'react';
import { useNavigate } from 'react-router-dom';

import { ExpandMore } from '@mui/icons-material';
import { DateTimePicker, renderTimeViewClock } from '@mui/x-date-pickers';
import CustomDivider from 'components/common/CustomDivider/CustomDivider';
import dayjs from 'dayjs';
import { BACKEND_URL } from 'services/backendUrl';
import { useImmer } from 'use-immer';
import { displaySnackbar } from 'utils/snackbar';
import ExerciseCreationCard from './creation/ExerciseCreationCard';

const minDate = dayjs().subtract(1, 'month');
const currentDate = dayjs();
const maxDate = dayjs().add(1, 'year');

export default function TrainingsCreationModal({ isOpenModal, setIsOpenModal }) {
    const navigate = useNavigate();
    const axiosPrivate = useAxiosPrivate();
    const [open, setOpen] = useImmer(false);
    const [mentor, setMentor] = useImmer(undefined);
    const [description, setDescription] = useImmer('');
    const [clients, setClients] = useImmer([]);
    const [selectedClient, setSelectedClient] = useImmer(undefined);
    const [availableExercises, setAvailableExercises] = useImmer([]);
    const [personalizedExercises, setPersonalizedExercises] = useImmer([]);

    const [filledExercises, setFilledExercises] = useImmer([]);
    const [preparedParameters, setPreparedParameters] = useImmer([]);
    const [preparedMetrics, setPreparedMetrics] = useImmer([]);
    // TODO: finish later
    const [preparedEquipments, setPreparedEquipments] = useImmer([]);

    const [startDate, setStartDate] = useImmer(currentDate);
    const [endDate, setEndDate] = useImmer(currentDate);
    const defaultTimeoutMsec = 2500;

    const handleOpen = () => setOpen(true);
    const handleClose = () => {
        setIsOpenModal(false);
        setOpen(false);
    };

    useEffect(() => {
        if (isOpenModal) {
            handleOpen();
        }
        if (!mentor) {
            fetchInfoAboutMentor();
        }
        if (isOpenModal && clients.length === 0) {
            fetchClientsByMentor();
        }
        if (availableExercises.length === 0) {
            fetchAllExercises();
        }
    }, [isOpenModal]);

    const fetchInfoAboutMentor = async () => {
        await axiosPrivate
            .get(`${BACKEND_URL}users/me`)
            .then((response) => {
                setMentor(response.data);
            })
            .catch((error) => {
                console.log(error);
            });
    };

    const fetchClientsByMentor = async () => {
        await axiosPrivate
            .get(`${BACKEND_URL}users/clients`)
            .then((response) => {
                setClients(response.data);
            })
            .catch((error) => {
                console.log(error);
            });
    };

    const fetchAllExercises = async () => {
        await axiosPrivate.get(`${BACKEND_URL}exercises`).then((response) => {
            setAvailableExercises(response.data);
        });
    };

    const handleClickOnButton = (event) => {
        event.preventDefault();

        const updatedEquipments = preparedEquipments.map((equipments, index) => {
            return equipments.map((equipment, innerIndex) => {
                return {
                    id: equipment.id,
                    name: equipment.name,
                    media: equipment.media,
                    properties_personal: equipment.properties_template.map((property) => {
                        // TODO: finish this logic later
                        return { ...property, value: null };
                    })
                };
            });
        });

        const updatedExercises = filledExercises.map((exercise, index) => {
            return {
                id: exercise.id,
                name: exercise.name,
                media: exercise.media,
                description: exercise.description,
                parameters_personal: preparedParameters[index],
                metrics_personal: preparedMetrics[index],
                equipments_personal: updatedEquipments[index]
            };
        });

        const creationTraining = {
            client: selectedClient,
            mentor: mentor,
            note: description,
            date_time_start: startDate,
            // FIXME: backend problem with finish date
            date_time_finish: startDate,
            date_plan: endDate,
            exercise_personal: updatedExercises
        };
        axiosPrivate
            .post(`${BACKEND_URL}trainings`, creationTraining)
            .then((response) => {
                if (response.status === 201) {
                    const message = 'Тренировка успешно создана!';
                    const snackbarKey = displaySnackbar(message, 'success');
                }
                const timerId = setTimeout(() => window.location.reload(), 1600);
            })
            .catch((error) => {
                console.log(error);
            });
    };

    const handleClickOnAddNewExerciseButton = () => {
        // TODO: handle last element in array
        setPersonalizedExercises((prev) => [...prev, {}]);
    };

    // TODO: создавать multi-choice элементы в отдельных аккордионах через кнопку добавить. Из dropdown списка выбирается базовый тип, далее он персонализируется в доп. поле ввода
    return (
        <PreparedDialog
            open={open}
            onClose={handleClose}
            scroll="body"
            aria-labelledby="scroll-dialog-title"
            aria-describedby="scroll-dialog-description">
            <DialogContent
                dividers={scroll === 'paper'}
                className="flex justify-center items-center p-0 w-full">
                <div className="flex flex-col items-center bg-[#e3f2fc] shadow-md rounded-lg gap-2 py-3 w-full">
                    <span className="font-bold text-[var(--color-unused-text-gray)] text-3xl text-center px-4">
                        Создание тренировки
                    </span>
                    <Form
                        onSubmit={handleClickOnButton}
                        className="flex flex-col justify-start items-center gap-3 px-3">
                        <FormControl className="w-full">
                            <InputLabel id="client-select-label">Клиент</InputLabel>
                            <Select
                                labelId="client-select-component"
                                id="client-select-component"
                                value={selectedClient}
                                label="Клиент"
                                onChange={(e) => {
                                    setSelectedClient(e.target.value);
                                }}>
                                {clients.map((client) => {
                                    return (
                                        <MenuItem value={client}>
                                            {`${client.first_name} ${client.last_name}`}
                                        </MenuItem>
                                    );
                                })}
                            </Select>
                        </FormControl>
                        <InputForm
                            className="text-lg lg:text-xl bg-[#fefefe] p-2 border-solid border-[#a3a9b2] border-2 rounded-md w-full"
                            type="text"
                            placeholder="Описание тренировки"
                            onChange={(e) => setDescription(e.target.value)}
                        />
                        <LocalizationProvider dateAdapter={AdapterDayjs} adapterLocale="ru">
                            <DateTimePicker
                                label="Дата начала тренировки"
                                value={startDate}
                                minDate={minDate}
                                maxDate={maxDate}
                                viewRenderers={{
                                    hours: renderTimeViewClock,
                                    minutes: renderTimeViewClock
                                }}
                                onChange={(value) => setStartDate(value)}
                                className="w-full"
                            />

                            <DateTimePicker
                                label="Дата окончания тренировки (план)"
                                minDate={startDate}
                                maxDate={maxDate}
                                value={endDate}
                                viewRenderers={{
                                    hours: renderTimeViewClock,
                                    minutes: renderTimeViewClock
                                }}
                                onChange={(value) => setEndDate(value)}
                                className="w-full"
                            />
                        </LocalizationProvider>

                        <Accordion
                            sx={{ p: 0 }}
                            className="text-2xl md:text-lg bg-[var(--color-contrast-blue)] text-[var(--color-form-white)]">
                            <AccordionSummary
                                sx={{ px: 2 }}
                                expandIcon={<ExpandMore />}
                                aria-controls="panel1-content"
                                id="panel1-header">
                                <Box px={0}>
                                    <p className="text-2xl md:text-lg text-[var(--color-form-white)]">
                                        Программа тренировки
                                    </p>
                                </Box>
                            </AccordionSummary>
                            <AccordionDetails>
                                <CustomDivider />
                                <p className="text-sm">Добавьте упражнения к тренировке</p>
                                {personalizedExercises.map((exercise) => {
                                    return (
                                        <div className="my-1">
                                            <ExerciseCreationCard
                                                exercise={exercise}
                                                availableExercises={availableExercises}
                                                setFilledExercises={setFilledExercises}
                                                setPreparedParameters={setPreparedParameters}
                                                setPreparedMetrics={setPreparedMetrics}
                                                setPreparedEquipments={setPreparedEquipments}
                                            />
                                        </div>
                                    );
                                })}
                                <Button
                                    color="inherit"
                                    variant="outlined"
                                    className="mt-2 w-full"
                                    onClick={handleClickOnAddNewExerciseButton}>
                                    Выбрать новое упражнение
                                </Button>
                            </AccordionDetails>
                        </Accordion>

                        <Button
                            className="flex flex-row justify-center font-normal rounded-xl border-solid border-2 py-2 px-4 lg:px-10 hover:text-[var(--color-form-white)] hover:border-[var(--color-thymeleaf)] hover:bg-[var(--color-thymeleaf)] text-[var(--color-form-white)] bg-[var(--color-thymeleaf)] border-[var(--color-thymeleaf)] w-full text-xl"
                            onClick={handleClickOnButton}>
                            Создать тренировку
                        </Button>
                    </Form>
                </div>
            </DialogContent>
        </PreparedDialog>
    );
}
